# Generated by Django 4.0.7 on 2022-09-03 18:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("killtracker", "0008_copy_data_to_new_structure"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="evekillmailvictim",
            name="alliance",
        ),
        migrations.RemoveField(
            model_name="evekillmailvictim",
            name="character",
        ),
        migrations.RemoveField(
            model_name="evekillmailvictim",
            name="corporation",
        ),
        migrations.RemoveField(
            model_name="evekillmailvictim",
            name="faction",
        ),
        migrations.RemoveField(
            model_name="evekillmailvictim",
            name="killmail",
        ),
        migrations.RemoveField(
            model_name="evekillmailvictim",
            name="ship_type",
        ),
        migrations.RemoveField(
            model_name="evekillmailzkb",
            name="killmail",
        ),
        migrations.DeleteModel(
            name="EveKillmailPosition",
        ),
        migrations.DeleteModel(
            name="EveKillmailVictim",
        ),
        migrations.DeleteModel(
            name="EveKillmailZkb",
        ),
    ]
