# Generated by Django 4.0.7 on 2022-09-03 17:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0007_evetype_description"),
        ("killtracker", "0006_evetypeplus"),
    ]

    operations = [
        migrations.AddField(
            model_name="evekillmail",
            name="alliance",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="character",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="corporation",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="damage_taken",
            field=models.BigIntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="faction",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="fitted_value",
            field=models.FloatField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="hash",
            field=models.CharField(blank=True, default="", max_length=64),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="is_awox",
            field=models.BooleanField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="is_npc",
            field=models.BooleanField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="is_solo",
            field=models.BooleanField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="location_id",
            field=models.PositiveIntegerField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="position_x",
            field=models.FloatField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="position_y",
            field=models.FloatField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="position_z",
            field=models.FloatField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="ship_type",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="total_value",
            field=models.FloatField(blank=True, db_index=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="evekillmail",
            name="zkb_points",
            field=models.PositiveIntegerField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
    ]
